@@ -74,21 +74,75 @@ t_protein_feature_match =marshalable_object("t_protein_feature_match",
                                               " hit_end"    +
                                               " hit_name")
 
-#_________________________________________________________________________________________
 
-#   link_protein_feature_matches_to_prot_id
+#_____________________________________________________________________________________
+#
+#   get_prot_id_to_features
+#_____________________________________________________________________________________
+def get_prot_id_to_features(matches_by_type, protein_features = None, remove_overlapping = False):
+    """
+    Index allowing quick lookup of which features are in which protein
+    """
+    prot_id_to_features = defaultdict(list)
+    #
+    #   index copy of data
+    #
+    for feature_type in matches_by_type:
+        for prot_id, features in matches_by_type[feature_type].iteritems():
+            prot_id_to_features[prot_id].append((feature_type, list(features)))
+    #
+    #   Use domain name rather than pfam / smart id
+    #
+    if protein_features:
+        for prot_id, data in prot_id_to_features.iteritems():
+            for feature_type, features in data:
+                for i in range(len(features)):
+                    feature = features[i]
+                    if feature.hit_name in protein_features:
+                        features[i] = t_protein_feature_match(*
+                                            (features[i]._as_tuple()[:-1])
+                                            + (protein_features[feature.hit_name].display_label,))
+    #
+    #   Remove overlapping, keeping hits with lowest e-value
+    #
+    if remove_overlapping:
+        for prot_id, data in prot_id_to_features.iteritems():
+            for i in range(len(data)):
+                feature_type, features = data[i]
+                features = sorted(features, key = lambda x: (x.seq_beg, x.seq_end))
+                non_overlapping_features = [features[0]]
+                for f2 in features[1:]:
+                    f1 = non_overlapping_features[-1]
+                    # not overlapping, add and continue
+                    if f1.seq_end < f2.seq_end:
+                        non_overlapping_features.append(f2)
+                        continue
+                    if f1.evalue < f2.evalue:
+                        continue
+                    non_overlapping_features[-1] = f2
+                data[i] = feature_type, non_overlapping_features
+    return prot_id_to_features
+
 
 #_________________________________________________________________________________________
-def link_protein_feature_matches_to_prot_id (matches_by_type):
+
+#   get_protein_features_to_prot_id
+
+#_________________________________________________________________________________________
+def get_protein_features_to_prot_id (matches_by_type, protein_features = None):
     """
     Allow quick lookup of which proteins have a particular hit
     """
     hit_name_to_prot_id  = defaultdict(set)
 
     for feature_type in matches_by_type:
-        for prot_id, features in matches_by_type[feature_type]:
+        for prot_id, features in matches_by_type[feature_type].iteritems():
             for feature in features:
-                hit_name_to_prot_id[feature.hit_name].add(prot_id)
+                if protein_features and feature.hit_name in protein_features:
+                    hit_name = protein_features[feature.hit_name].display_label
+                else:
+                    hit_name = feature.hit_name
+                hit_name_to_prot_id[hit_name].add(prot_id)
 
     return hit_name_to_prot_id
 
